using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Messaging;
using System.Workflow.Runtime.Hosting.Web;

using Workflow.RuntimeServices;
using Workflow.Workflows;

namespace Workflow.Web
{
	[Serializable]
	public class WorkflowControllerHandler : IHttpHandler
	{
        WorkflowRuntime workflowRuntime = null;
        GetNameService getNameService = null;
        GetEmailService getEmailService = null;
        SendDataService sendDataService = null;

		public bool IsReusable
		{
			get { return false; }
		}

		public void ProcessRequest(HttpContext context)
		{
            InitWorkflowRuntime();

            // see if there is a workflow guid in the query string
			if (context.Request.QueryString["id"] == null)
			{
                // no guid present, start a new workflow
				Guid guid = Guid.NewGuid();
				getNameService.instanceId = guid;
				getEmailService.instanceId = guid;

				workflowRuntime.StartWorkflow(typeof(Workflows.PersonalInfoWorkflow), guid);
			}
			else
			{
                // parse the guid and continue the flow
				Guid guid = new Guid(context.Request.QueryString["id"]);

                getNameService.instanceId = guid;
				getEmailService.instanceId = guid;

                switch (context.Request.QueryString["action"])
                {
                    case "getName":
                        SubmitName(context.Request.QueryString["firstName"], context.Request.QueryString["lastName"]);
                        break;
                    case "getEmail":
                        SubmitEmail(context.Request.QueryString["email"]);
                        break;
                }
			}
		}

        private void SubmitName(string firstName, string lastName)
        {
            getNameService.SubmitName(firstName, lastName);
        }

        private void SubmitEmail(string email)
        {
            getEmailService.SubmitEmail(email);
        }

		private void nameService_NameRequested(object sender, WorkflowMessageEventArgs e)
		{
            // the name was requested from the workflow
            // redirect the user to the name entry page
            try
            {
                HttpContext.Current.Response.Redirect("~/GetName.aspx?id=" + e.InstanceId.ToString(), false);
            }
            finally { }
		}

		private void emailService_EmailRequested(object sender, WorkflowMessageEventArgs e)
		{
            // the email was requested from the workflow
            // redirect the user to the email entry page
            try
            {
                HttpContext.Current.Response.Redirect("~/GetEmail.aspx?id=" + e.InstanceId.ToString(), false);
            }
            finally { }
        }

        private void sendDataService_DataReceived(object sender, DataReceivedEventArgs e)
        {
            // the personal data was sent from the workflow
            // redirect the user to the display page
            try
            {
                HttpContext.Current.Response.Redirect(
                    "~/ShowData.aspx?f=" + e.FirstName +
                    "&l=" + e.LastName +
                    "&e=" + e.EmailAddress,
                    false);
            }
            finally { }
        }

        private void InitWorkflowRuntime()
        {
            // obtain a reference to the workflow runtime
            workflowRuntime = WorkflowRequestContext.Current.WorkflowRuntime;

            // setup communications through the service classes defined in the WorkflowClassLibrary project

            getNameService = (GetNameService)workflowRuntime.GetService(typeof(GetNameService));
            getNameService.NameRequested += new EventHandler<WorkflowMessageEventArgs>(nameService_NameRequested);

            getEmailService = (GetEmailService)workflowRuntime.GetService(typeof(GetEmailService));
            getEmailService.EmailRequested += new EventHandler<WorkflowMessageEventArgs>(emailService_EmailRequested);

            sendDataService = (SendDataService)workflowRuntime.GetService(typeof(SendDataService));
            sendDataService.DataReceived += new EventHandler<DataReceivedEventArgs>(sendDataService_DataReceived);
        }
    }
}